// Licensed to Elasticsearch B.V. under one or more contributor
// license agreements. See the NOTICE file distributed with
// this work for additional information regarding copyright
// ownership. Elasticsearch B.V. licenses this file to you under
// the Apache License, Version 2.0 (the "License"); you may
// not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing,
// software distributed under the License is distributed on an
// "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
// KIND, either express or implied.  See the License for the
// specific language governing permissions and limitations
// under the License.


// Code generated from the elasticsearch-specification DO NOT EDIT.
// https://github.com/elastic/elasticsearch-specification/tree/4316fc1aa18bb04678b156f23b22c9d3f996f9c9


// Evicts users from the user cache. Can completely clear the cache or evict
// specific users.
package clearcachedrealms

import (
	gobytes "bytes"
	"context"
	"errors"
	"fmt"
	"io"
	"io/ioutil"
	"net/http"
	"net/url"
	"strings"

	"github.com/elastic/elastic-transport-go/v8/elastictransport"
)

const (
	realmsMask = iota + 1
)

// ErrBuildPath is returned in case of missing parameters within the build of the request.
var ErrBuildPath = errors.New("cannot build path, check for missing path parameters")

type ClearCachedRealms struct {
	transport elastictransport.Interface

	headers http.Header
	values  url.Values
	path    url.URL

	buf *gobytes.Buffer

	paramSet int

	realms string
}

// NewClearCachedRealms type alias for index.
type NewClearCachedRealms func(realms string) *ClearCachedRealms

// NewClearCachedRealmsFunc returns a new instance of ClearCachedRealms with the provided transport.
// Used in the index of the library this allows to retrieve every apis in once place.
func NewClearCachedRealmsFunc(tp elastictransport.Interface) NewClearCachedRealms {
	return func(realms string) *ClearCachedRealms {
		n := New(tp)

		n.Realms(realms)

		return n
	}
}

// Evicts users from the user cache. Can completely clear the cache or evict
// specific users.
//
// https://www.elastic.co/guide/en/elasticsearch/reference/current/security-api-clear-cache.html
func New(tp elastictransport.Interface) *ClearCachedRealms {
	r := &ClearCachedRealms{
		transport: tp,
		values:    make(url.Values),
		headers:   make(http.Header),
		buf:       gobytes.NewBuffer(nil),
	}

	return r
}

// HttpRequest returns the http.Request object built from the
// given parameters.
func (r *ClearCachedRealms) HttpRequest(ctx context.Context) (*http.Request, error) {
	var path strings.Builder
	var method string
	var req *http.Request

	var err error

	r.path.Scheme = "http"

	switch {
	case r.paramSet == realmsMask:
		path.WriteString("/")
		path.WriteString("_security")
		path.WriteString("/")
		path.WriteString("realm")
		path.WriteString("/")
		path.WriteString(url.PathEscape(r.realms))
		path.WriteString("/")
		path.WriteString("_clear_cache")

		method = http.MethodPost
	}

	r.path.Path = path.String()
	r.path.RawQuery = r.values.Encode()

	if r.path.Path == "" {
		return nil, ErrBuildPath
	}

	if ctx != nil {
		req, err = http.NewRequestWithContext(ctx, method, r.path.String(), r.buf)
	} else {
		req, err = http.NewRequest(method, r.path.String(), r.buf)
	}

	req.Header.Set("accept", "application/vnd.elasticsearch+json;compatible-with=8")

	if err != nil {
		return req, fmt.Errorf("could not build http.Request: %w", err)
	}

	return req, nil
}

// Do runs the http.Request through the provided transport.
func (r ClearCachedRealms) Do(ctx context.Context) (*http.Response, error) {
	req, err := r.HttpRequest(ctx)
	if err != nil {
		return nil, err
	}

	res, err := r.transport.Perform(req)
	if err != nil {
		return nil, fmt.Errorf("an error happened during the ClearCachedRealms query execution: %w", err)
	}

	return res, nil
}

// IsSuccess allows to run a query with a context and retrieve the result as a boolean.
// This only exists for endpoints without a request payload and allows for quick control flow.
func (r ClearCachedRealms) IsSuccess(ctx context.Context) (bool, error) {
	res, err := r.Do(ctx)

	if err != nil {
		return false, err
	}
	io.Copy(ioutil.Discard, res.Body)
	err = res.Body.Close()
	if err != nil {
		return false, err
	}

	if res.StatusCode >= 200 && res.StatusCode < 300 {
		return true, nil
	}

	return false, nil
}

// Header set a key, value pair in the ClearCachedRealms headers map.
func (r *ClearCachedRealms) Header(key, value string) *ClearCachedRealms {
	r.headers.Set(key, value)

	return r
}

// Realms Comma-separated list of realms to clear
// API Name: realms
func (r *ClearCachedRealms) Realms(v string) *ClearCachedRealms {
	r.paramSet |= realmsMask
	r.realms = v

	return r
}

// Usernames Comma-separated list of usernames to clear from the cache
// API name: usernames
func (r *ClearCachedRealms) Usernames(value string) *ClearCachedRealms {
	r.values.Set("usernames", value)

	return r
}
